{% load i18n %}{% load staticfiles %}{% get_current_language as LANGUAGE_CODE %}<!DOCTYPE html>
<html{% if LANGUAGE_CODE %} lang="{{LANGUAGE_CODE}}"{% endif %}>
    <head>
        <meta charset="utf-8">
        <!--[if IE]><meta http-equiv="X-UA-Compatible" content="IE=edge" /><![endif]-->
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>{% block title %}{% trans "Central Authentication Service"  %}{% endblock %}</title>
        <link href="{{settings.CAS_COMPONENT_URLS.bootstrap3_css}}" rel="stylesheet">
        <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
        <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
        <!--[if lt IE 9]>
        <script src="{{settings.CAS_COMPONENT_URLS.html5shiv}}"></script>
        <script src="{{settings.CAS_COMPONENT_URLS.respond}}"></script>
        <![endif]-->
        {% if settings.CAS_FAVICON_URL %}<link rel="shortcut icon" href="{{settings.CAS_FAVICON_URL}}" />{% endif %}
        <link href="{% static "cas_server/styles.css" %}" rel="stylesheet">
    </head>
    <body>
      <div id="wrap">
        <div class="container">
            {% if auto_submit %}<noscript>{% endif %}
            <div class="row">
              <div class="col-lg-12 col-md-12 col-sm-12 col-xs-12">
                <h1 id="app-name">
                    {% if settings.CAS_LOGO_URL %}<img src="{{settings.CAS_LOGO_URL}}" alt="cas-logo" />{% endif %}
                    {% trans "Central Authentication Service" %}</h1>
              </div>
            </div>
            {% if auto_submit %}</noscript>{% endif %}
            <div class="row">
            <div class="col-lg-3 col-md-3 col-sm-2 col-xs-12"></div>
            <div class="col-lg-6 col-md-6 col-sm-8 col-xs-12">
            {% if auto_submit %}<noscript>{% endif %}
            {% for msg in CAS_INFO_RENDER %}
              <div class="alert alert-{{msg.type}}{% if msg.discardable %} alert-dismissable{% endif %}">
                {% if msg.discardable %}<button type="button" class="close" data-dismiss="alert" aria-hidden="true" id="info-{{msg.name}}">&#215;</button>{% endif %}
                <p>{{msg.message}}</p>
              </div>
            {% endfor %}
            {% if settings.CAS_NEW_VERSION_HTML_WARNING and upgrade_available %}
              <div class="alert alert-info alert-dismissable">
                <button type="button" class="close" data-dismiss="alert" aria-hidden="true" id="alert-version">&#215;</button>
                <p>{% blocktrans %}A new version of the application is available. This instance runs {{VERSION}} and the last version is {{LAST_VERSION}}. Please consider upgrading.{% endblocktrans %}</p>
              </div>
            {% endif %}
            {% block ante_messages %}{% endblock %}
            {% for message in messages %}
                <div {% spaceless %}
                    {% if message.level == message_levels.DEBUG %}
                        class="alert alert-warning"
                    {% elif message.level == message_levels.INFO %}
                        class="alert alert-info"
                    {% elif message.level == message_levels.SUCCESS %}
                        class="alert alert-success"
                    {% elif message.level == message_levels.WARNING %}
                        class="alert alert-warning"
                    {% else %}
                        class="alert alert-danger"
                    {% endif %}
                {% endspaceless %}>
                    <p>{{message|safe}}</p>
                </div>
            {% endfor %}
            {% if auto_submit %}</noscript>{% endif %}
            {% block content %}{% endblock %}
            </div>
            <div class="col-lg-3 col-md-3 col-sm-2 col-xs-0"></div>
            </div>
        </div> <!-- /container -->
      </div>
      <div style="clear: both;"></div>
      {% if settings.CAS_SHOW_POWERED %}
      <div id="footer">
          <p><a class="text-muted" href="https://pypi.python.org/pypi/django-cas-server">django-cas-server powered</a></p>
      </div>
      {% endif %}
      <script src="{{settings.CAS_COMPONENT_URLS.jquery}}"></script>
      <script src="{{settings.CAS_COMPONENT_URLS.bootstrap3_js}}"></script>
      <script src="{% static "cas_server/functions.js" %}"></script>
      <script type="text/javascript">
{% if settings.CAS_NEW_VERSION_HTML_WARNING and upgrade_available %}
discard_and_remember("#alert-version", "cas-alert-version", "{{LAST_VERSION}}");
{% endif %}
{% for msg in CAS_INFO_RENDER %}
{% if msg.discardable %}
discard_and_remember("#info-{{msg.name}}", "cas-info-{{msg.name}}", "{{msg.hash}}");
{% endif %}
{% endfor %}
{% block javascript_inline %}{% endblock %}
</script>
      {% block javascript %}{% endblock %}
    </body>
</html>
<!--
Powered by django-cas-server version {{VERSION}}

Pypi: https://pypi.python.org/pypi/django-cas-server
github: https://github.com/nitmir/django-cas-server
-->
