{% extends "cas_server/base.html" %}
{% load staticfiles %}
{% load i18n %}

{% block content %}
      <form class="form-signin" method="post">
{% csrf_token %}
{% include "cas_server/form.html" %}
<button class="btn btn-primary btn-block btn-lg" type="submit">{% trans "Connect to the service" %}</button>
      </form>
{% endblock %}
