{% load cas_server %}
{% for error in form.non_field_errors %}
<div class="alert alert-danger alert-dismissable">
  <button type="button" class="close" data-dismiss="alert" aria-hidden="true">&#215;</button>
  {{error}}
</div>
{% endfor %}
{% for field in form %}{% if not field|is_hidden %}
<div class="form-group{% spaceless %}
  {% if not form.non_field_errors %}
    {% if field.errors %} has-error
    {% elif form.cleaned_data %} has-success
    {% endif %}
  {% endif %}"
{% endspaceless %}>{% spaceless %}
  {% if field|is_checkbox %}
    <div class="checkbox"><label for="{{field.auto_id}}">{{field}}{{field.label}}</label></div>
  {% else %}
    <label class="control-label" for="{{field.auto_id}}">{{field.label}}</label>
    {{field}}
  {% endif %}
  {% for error in field.errors %}
    <span class="help-block">{{error}}</span>
  {% endfor %}
{% endspaceless %}</div>
{% else %}{{field}}{% endif %}{% endfor %}
