# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-07-23 22:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cas_server', '0006_auto_20160706_1727'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='federateduser',
            name='attributs',
        ),
        migrations.RemoveField(
            model_name='proxygrantingticket',
            name='attributs',
        ),
        migrations.RemoveField(
            model_name='proxyticket',
            name='attributs',
        ),
        migrations.RemoveField(
            model_name='serviceticket',
            name='attributs',
        ),
        migrations.AddField(
            model_name='federateduser',
            name='_attributs',
            field=models.TextField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='proxygrantingticket',
            name='_attributs',
            field=models.TextField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='proxyticket',
            name='_attributs',
            field=models.TextField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='serviceticket',
            name='_attributs',
            field=models.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name='federatediendityprovider',
            name='suffix',
            field=models.CharField(help_text='Suffix append to backend CAS returned username: ``returned_username`` @ ``suffix``.', max_length=30, unique=True, verbose_name='suffix'),
        ),
    ]
