# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-06 17:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cas_server', '0005_auto_20160616_1018'),
    ]

    operations = [
        migrations.AlterField(
            model_name='federatediendityprovider',
            name='cas_protocol_version',
            field=models.CharField(choices=[(b'1', b'CAS 1.0'), (b'2', b'CAS 2.0'), (b'3', b'CAS 3.0'), (b'CAS_2_SAML_1_0', b'SAML 1.1')], default=b'3', help_text='Version of the CAS protocol to use when sending requests the the backend CAS.', max_length=30, verbose_name='CAS protocol version'),
        ),
        migrations.AlterField(
            model_name='federatediendityprovider',
            name='display',
            field=models.BooleanField(default=True, help_text='Display the provider on the login page.', verbose_name='display'),
        ),
        migrations.AlterField(
            model_name='federatediendityprovider',
            name='pos',
            field=models.IntegerField(default=100, help_text='Position of the identity provider on the login page. Identity provider are sorted using the (position, verbose name, suffix) attributes.', verbose_name='position'),
        ),
        migrations.AlterField(
            model_name='federatediendityprovider',
            name='suffix',
            field=models.CharField(help_text='Suffix append to backend CAS returner username: ``returned_username`` @ ``suffix``.', max_length=30, unique=True, verbose_name='suffix'),
        ),
        migrations.AlterField(
            model_name='federatediendityprovider',
            name='verbose_name',
            field=models.CharField(help_text='Name for this identity provider displayed on the login page.', max_length=255, verbose_name='verbose name'),
        ),
    ]
